@@ -1,12 +1,12 @@
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Tuple
 from dataclasses import asdict
 import time
 from datetime import datetime, timezone
+import math
 
-from ..core.base import BaseProcessor, ProcessingContext, ProcessingResult, ConfigProtocol, ResultFormat
+from ..core.base import BaseProcessor, ProcessingContext, ProcessingResult, ConfigProtocol
 from ..utils import (
     filter_by_confidence,
-    filter_by_categories,
     apply_category_mapping,
     count_objects_by_category,
     count_objects_in_zones,
@@ -14,46 +14,38 @@ from ..utils import (
     match_results_structure,
     bbox_smoothing,
     BBoxSmoothingConfig,
-    BBoxSmoothingTracker
+    BBoxSmoothingTracker,
+    get_bbox_center,
+    point_in_polygon
 )
 from dataclasses import dataclass, field
-from ..core.config import BaseConfig, AlertConfig, ZoneConfig
+from ..core.config import BaseConfig, AlertConfig
 
 @dataclass
 class AbandonedObjectConfig(BaseConfig):
     """Configuration for abandoned object detection use case."""
-    enable_smoothing: bool = False
-    centroid_threshold: float = 10.0
+    enable_smoothing: bool = True
+    centroid_threshold: float = 15.0
+    proximity_threshold: float = 50.0  # New: Distance threshold for person proximity
     smoothing_algorithm: str = "observability"
     smoothing_window_size: int = 20
     smoothing_cooldown_frames: int = 5
     smoothing_confidence_range_factor: float = 0.5
     confidence_threshold: float = 0.4
-    usecase_categories: List[str] = field(
-        default_factory=lambda: ['backpack', 'handbag', 'suitcase']
-    )
-    target_categories: List[str] = field(
-        default_factory=lambda: ['backpack', 'handbag', 'suitcase']
-    )
+    stationary_threshold_frames: int = 30
+    usecase_categories: List[str] = field(default_factory=lambda: ['backpack', 'handbag', 'suitcase'])
+    target_categories: List[str] = field(default_factory=lambda: ['backpack', 'handbag', 'suitcase'])
     alert_config: Optional[AlertConfig] = None
+    zone_config: Optional[Dict[str, Dict[str, List[List[float]]]]] = None
     index_to_category: Optional[Dict[int, str]] = field(
-        default_factory=lambda: {
-            24: "backpack",
-            26: "handbag",
-            28: "suitcase"
-        }
+        default_factory=lambda: {24: "backpack", 26: "handbag", 28: "suitcase"}
     )
-    stationary_threshold_frames: int = 30  # Reduced to 1 for immediate detection in testing
     person_category: str = "person"
     person_index: int = 0
 
 class AbandonedObjectDetectionUseCase(BaseProcessor):
-    CATEGORY_DISPLAY = {
-        "backpack": "backpack",
-        "handbag": "handbag",
-        "suitcase": "suitcase"
-    }
-    
+    CATEGORY_DISPLAY = {"backpack": "Backpack", "handbag": "Handbag", "suitcase": "Suitcase"}
+
     def __init__(self):
         super().__init__("abandoned_object_detection")
         self.category = "security"
@@ -71,8 +63,12 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
         self._track_merge_time_window: float = 7.0
         self._ascending_alert_list: List[int] = []
         self.current_incident_end_timestamp: str = "N/A"
-        self._stationary_tracks: Dict[Any, Dict[str, Any]] = {}  # Track stationary objects
-        self._person_bboxes: List[Dict] = []  # Store person bboxes for overlap check
+        self._stationary_tracks: Dict[Any, Dict[str, Any]] = {}
+        self._zone_current_track_ids: Dict[str, set] = {}
+        self._zone_total_track_ids: Dict[str, set] = {}
+        self._zone_current_counts: Dict[str, int] = {}
+        self._zone_total_counts: Dict[str, int] = {}
+        self.start_timer = None
 
     def process(self, data: Any, config: ConfigProtocol, context: Optional[ProcessingContext] = None,
                 stream_info: Optional[Dict[str, Any]] = None) -> ProcessingResult:
@@ -89,12 +85,10 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
         processed_data = filter_by_confidence(data, config.confidence_threshold)
         if config.index_to_category:
             processed_data = apply_category_mapping(processed_data, config.index_to_category)
-        processed_data = [d for d in processed_data if d.get('category') in self.target_categories or d.get('category') == config.person_category]
 
-        # Store person bboxes for overlap check
-        self._person_bboxes = [d.get('bounding_box') for d in processed_data if d.get('category') == config.person_category]
+        person_dets = [d for d in processed_data if d.get('category') == config.person_category]
+        object_dets = [d for d in processed_data if d.get('category') in self.target_categories]
 
-        # Apply bbox smoothing
         if config.enable_smoothing:
             if self.smoothing_tracker is None:
                 smoothing_config = BBoxSmoothingConfig(
@@ -108,7 +102,6 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
                 self.smoothing_tracker = BBoxSmoothingTracker(smoothing_config)
             processed_data = bbox_smoothing(processed_data, self.smoothing_tracker.config, self.smoothing_tracker)
 
-        # Advanced tracking
         try:
             from ..advanced_tracker import AdvancedTracker
             from ..advanced_tracker.config import TrackerConfig
@@ -119,9 +112,9 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
         except Exception as e:
             self.logger.warning(f"AdvancedTracker failed: {e}")
 
-        # Check for stationary objects
-        abandoned_data = self._check_stationary_objects(processed_data, config)
-
+        person_dets = [d for d in processed_data if d.get('category') == config.person_category]
+        object_dets = [d for d in processed_data if d.get('category') in self.target_categories]
+        abandoned_data = self._check_abandoned_objects(object_dets, person_dets, config)
         self._update_tracking_state(abandoned_data)
         self._total_frame_counter += 1
 
@@ -136,13 +129,21 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
         counting_summary = self._count_categories(abandoned_data, config)
         total_counts = self.get_total_counts()
         counting_summary['total_counts'] = total_counts
-        alerts = self._check_alerts(counting_summary, frame_number, config)
+
+        zone_analysis = {}
+        if config.zone_config and config.zone_config['zones']:
+            frame_data = object_dets
+            zone_analysis = count_objects_in_zones(frame_data, config.zone_config['zones'])
+            if zone_analysis:
+                zone_analysis = self._update_zone_tracking(zone_analysis, abandoned_data, config)
+
+        alerts = self._check_alerts(counting_summary, zone_analysis, frame_number, config)
         predictions = self._extract_predictions(abandoned_data)
 
-        incidents_list = self._generate_incidents(counting_summary, alerts, config, frame_number, stream_info)
-        tracking_stats_list = self._generate_tracking_stats(counting_summary, alerts, config, frame_number, stream_info)
-        business_analytics_list = self._generate_business_analytics(counting_summary, alerts, config, stream_info, is_empty=True)
-        summary_list = self._generate_summary(counting_summary, incidents_list, tracking_stats_list, business_analytics_list, alerts)
+        incidents_list = self._generate_incidents(counting_summary, zone_analysis, alerts, config, frame_number, stream_info)
+        tracking_stats_list = self._generate_tracking_stats(counting_summary, zone_analysis, alerts, config, frame_number, stream_info)
+        business_analytics_list = self._generate_business_analytics(counting_summary, zone_analysis, alerts, config, stream_info, is_empty=True)
+        summary_list = self._generate_summary(counting_summary, zone_analysis, incidents_list, tracking_stats_list, business_analytics_list, alerts)
 
         incidents = incidents_list[0] if incidents_list else {}
         tracking_stats = tracking_stats_list[0] if tracking_stats_list else {}
@@ -153,6 +154,7 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
             "tracking_stats": tracking_stats,
             "business_analytics": business_analytics,
             "alerts": alerts,
+            "zone_analysis": zone_analysis,
             "human_text": summary}
         }
 
@@ -165,57 +167,70 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
         )
         return result
 
-    def _check_stationary_objects(self, data: List[Dict], config: AbandonedObjectConfig) -> List[Dict]:
-        """Check if objects are stationary and not overlapping with persons."""
+    def _check_abandoned_objects(self, object_dets: List[Dict], person_dets: List[Dict], config: AbandonedObjectConfig) -> List[Dict]:
         abandoned_data = []
         current_time = time.time()
 
-        for det in data:
-            if det.get('category') not in self.target_categories:
-                continue
+        for det in object_dets:
             track_id = det.get('track_id')
             bbox = det.get('bounding_box')
             if not track_id or not bbox:
                 continue
 
-            # Calculate centroid
             centroid = self._calculate_centroid(bbox)
+            in_zone = self._is_in_zone(bbox, config.zone_config['zones'] if config.zone_config else None)
+            is_near_person = False
+
+            # Proximity check: Calculate distance to nearest person's centroid
+            for person in person_dets:
+                person_bbox = person.get('bounding_box')
+                if person_bbox:
+                    person_centroid = self._calculate_centroid(person_bbox)
+                    distance = math.sqrt((centroid[0] - person_centroid[0])**2 + (centroid[1] - person_centroid[1])**2)
+                    if distance < config.proximity_threshold:
+                        is_near_person = True
+                        break
+
+            if is_near_person:
+                if track_id in self._stationary_tracks:
+                    del self._stationary_tracks[track_id]
+                continue
+
             if track_id not in self._stationary_tracks:
                 self._stationary_tracks[track_id] = {
                     'centroid': centroid,
                     'frame_count': 1,
                     'start_time': current_time,
-                    'bbox': bbox
+                    'bbox': bbox,
+                    'in_zone': in_zone
                 }
                 track_info = self._stationary_tracks[track_id]
-                # Custom logic: Check threshold even for new tracks (allows detection on frame 1 if threshold=1)
-                if track_info['frame_count'] >= config.stationary_threshold_frames:
-                    if not self._overlaps_with_person(bbox):
-                        det['category'] = 'abandoned_object'
-                        abandoned_data.append(det)
+                if track_info['frame_count'] >= config.stationary_threshold_frames and in_zone:
+                    det['category'] = 'abandoned_object'
+                    abandoned_data.append(det)
+                    self.logger.info(f"Marked as abandoned_object: {det}")
             else:
                 track_info = self._stationary_tracks[track_id]
                 prev_centroid = track_info['centroid']
                 track_info['frame_count'] += 1
                 track_info['bbox'] = bbox
+                track_info['in_zone'] = in_zone
 
-                # Custom logic: Relaxed stationary check - consider it stationary if distance < threshold or if no major movement detected
-                if self._is_centroid_stationary(centroid, prev_centroid, config.centroid_threshold * 1.5):  # Increased threshold for more leniency
-                    if track_info['frame_count'] >= config.stationary_threshold_frames:
-                        if not self._overlaps_with_person(bbox):
-                            det['category'] = 'abandoned_object'
-                            abandoned_data.append(det)
-                    track_info['centroid'] = centroid
+                if self._is_centroid_stationary(centroid, prev_centroid, config.centroid_threshold):
+                    if track_info['frame_count'] >= config.stationary_threshold_frames and in_zone:
+                        det['category'] = 'abandoned_object'
+                        abandoned_data.append(det)
+                        self.logger.info(f"Marked as abandoned_object: {det}")
                 else:
-                    # Custom logic: Instead of full reset, decrement frame_count slightly for minor movements
-                    track_info['frame_count'] = max(1, track_info['frame_count'] - 5)  # Gentle decrement
+                    track_info['frame_count'] = max(1, track_info['frame_count'] - 5)
                     track_info['centroid'] = centroid
                     track_info['start_time'] = current_time
 
+                self.logger.info(f"Object: confidence={det.get('confidence')}, category={det.get('category')}, track_id={track_id}, frame_count={track_info['frame_count']}, in_zone={in_zone}, centroid={centroid}, is_near_person={is_near_person}")
+
         return abandoned_data
 
     def _calculate_centroid(self, bbox: Dict) -> tuple:
-        """Calculate centroid of a bounding box."""
         if 'xmin' in bbox:
             x = (bbox['xmin'] + bbox['xmax']) / 2
             y = (bbox['ymin'] + bbox['ymax']) / 2
@@ -226,22 +241,67 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
             return (0, 0)
         return (x, y)
 
-    def _is_centroid_stationary(self, centroid: tuple, prev_centroid: tuple, threshold: float = 5.0) -> bool:
-        """Check if centroid movement is within threshold."""
-        threshold = threshold or self.config.centroid_threshold
+    def _is_centroid_stationary(self, centroid: tuple, prev_centroid: tuple, threshold: float) -> bool:
         x1, y1 = centroid
         x2, y2 = prev_centroid
         distance = ((x1 - x2) ** 2 + (y1 - y2) ** 2) ** 0.5
         return distance < threshold
 
-    def _overlaps_with_person(self, bbox: Dict) -> bool:
-        """Check if bbox overlaps with any person bbox."""
-        for person_bbox in self._person_bboxes:
-            if person_bbox and self._compute_iou(bbox, person_bbox) > 0.5:
+    def _is_in_zone(self, bbox: Dict, zones: Optional[Dict[str, List[List[float]]]]) -> bool:
+        if not zones:
+            return True
+        center = get_bbox_center(bbox)
+        for zone_name, zone_polygon in zones.items():
+            polygon_points = [(point[0], point[1]) for point in zone_polygon]
+            if point_in_polygon(center, polygon_points):
                 return True
         return False
 
-    def _check_alerts(self, summary: dict, frame_number: Any, config: AbandonedObjectConfig) -> List[Dict]:
+    def _update_zone_tracking(self, zone_analysis: Dict[str, Dict[str, int]], detections: List[Dict], config: AbandonedObjectConfig) -> Dict[str, Dict[str, Any]]:
+        if not zone_analysis or not config.zone_config or not config.zone_config['zones']:
+            return {}
+
+        enhanced_zone_analysis = {}
+        zones = config.zone_config['zones']
+        current_frame_zone_tracks = {}
+
+        for zone_name in zones.keys():
+            current_frame_zone_tracks[zone_name] = set()
+            if zone_name not in self._zone_current_track_ids:
+                self._zone_current_track_ids[zone_name] = set()
+            if zone_name not in self._zone_total_track_ids:
+                self._zone_total_track_ids[zone_name] = set()
+
+        for detection in detections:
+            track_id = detection.get("track_id")
+            if track_id is None:
+                continue
+            bbox = detection.get("bounding_box")
+            if not bbox:
+                continue
+            center_point = get_bbox_center(bbox)
+            for zone_name, zone_polygon in zones.items():
+                polygon_points = [(point[0], point[1]) for point in zone_polygon]
+                if point_in_polygon(center_point, polygon_points):
+                    current_frame_zone_tracks[zone_name].add(track_id)
+
+        for zone_name, zone_counts in zone_analysis.items():
+            current_tracks = current_frame_zone_tracks.get(zone_name, set())
+            self._zone_current_track_ids[zone_name] = current_tracks
+            self._zone_total_track_ids[zone_name].update(current_tracks)
+            self._zone_current_counts[zone_name] = len(current_tracks)
+            self._zone_total_counts[zone_name] = len(self._zone_total_track_ids[zone_name])
+            enhanced_zone_analysis[zone_name] = {
+                "current_count": self._zone_current_counts[zone_name],
+                "total_count": self._zone_total_counts[zone_name],
+                "current_track_ids": list(current_tracks),
+                "total_track_ids": list(self._zone_total_track_ids[zone_name]),
+                "original_counts": zone_counts
+            }
+
+        return enhanced_zone_analysis
+
+    def _check_alerts(self, summary: dict, zone_analysis: Dict, frame_number: Any, config: AbandonedObjectConfig) -> List[Dict]:
         def get_trend(data, lookback=900, threshold=0.6):
             window = data[-lookback:] if len(data) >= lookback else data
             if len(window) < 2:
@@ -253,7 +313,6 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
         frame_key = str(frame_number) if frame_number is not None else "current_frame"
         alerts = []
         total_detections = summary.get("total_count", 0)
-        total_counts_dict = summary.get("total_counts", {})
         per_category_count = summary.get("per_category_count", {})
 
         if not config.alert_config:
@@ -283,15 +342,15 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
                     })
         return alerts
 
-    def _generate_incidents(self, counting_summary: Dict, alerts: List, config: AbandonedObjectConfig,
+    def _generate_incidents(self, counting_summary: Dict, zone_analysis: Dict, alerts: List, config: AbandonedObjectConfig,
                            frame_number: Optional[int] = None, stream_info: Optional[Dict[str, Any]] = None) -> List[Dict]:
         incidents = []
         total_detections = counting_summary.get("total_count", 0)
         current_timestamp = self._get_current_timestamp_str(stream_info)
         camera_info = self.get_camera_info_from_stream(stream_info)
-        
+
         self._ascending_alert_list = self._ascending_alert_list[-900:] if len(self._ascending_alert_list) > 900 else self._ascending_alert_list
-        
+
         if total_detections > 0:
             level = "low"
             intensity = 5.0
@@ -371,7 +430,7 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
 
         return incidents
 
-    def _generate_tracking_stats(self, counting_summary: Dict, alerts: List, config: AbandonedObjectConfig,
+    def _generate_tracking_stats(self, counting_summary: Dict, zone_analysis: Dict, alerts: List, config: AbandonedObjectConfig,
                                 frame_number: Optional[int] = None, stream_info: Optional[Dict[str, Any]] = None) -> List[Dict]:
         camera_info = self.get_camera_info_from_stream(stream_info)
         tracking_stats = []
@@ -406,17 +465,53 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
 
         human_text_lines = [f"Tracking Statistics:"]
         human_text_lines.append(f"CURRENT FRAME @ {current_timestamp}")
-        for cat, count in per_category_count.items():
-            human_text_lines.append(f"\t{cat}: {count}")
+        if zone_analysis:
+            human_text_lines.append("\tZones (current):")
+            for zone_name, zone_data in zone_analysis.items():
+                current_count = 0
+                if isinstance(zone_data, dict):
+                    if "current_count" in zone_data:
+                        current_count = zone_data.get("current_count", 0)
+                    else:
+                        counts_dict = zone_data.get("original_counts") if isinstance(zone_data.get("original_counts"), dict) else zone_data
+                        current_count = counts_dict.get(
+                            "total",
+                            sum(v for v in counts_dict.values() if isinstance(v, (int, float)))
+                        )
+                human_text_lines.append(f"\t{zone_name}: {int(current_count)}")
+        else:
+            if any(count > 0 for count in per_category_count.values()):
+                for cat, count in per_category_count.items():
+                    if count > 0:
+                        human_text_lines.append(f"\t- {count} Abandoned Objects detected")
+                    else:
+                        human_text_lines.append(f"\t- No detections")
         human_text_lines.append(f"TOTAL SINCE {start_timestamp}")
-        for cat, count in total_counts_dict.items():
-            if count > 0:
-                human_text_lines.append(f"\t{cat}: {count}")
-        human_text_lines.append(f"Alerts: {alerts[0].get('settings', {}) if alerts else 'None'} sent @ {current_timestamp}")
+        if zone_analysis:
+            human_text_lines.append("\tZones (total):")
+            for zone_name, zone_data in zone_analysis.items():
+                total_count = 0
+                if isinstance(zone_data, dict):
+                    if "total_count" in zone_data and isinstance(zone_data.get("total_count"), (int, float)):
+                        total_count = zone_data.get("total_count", 0)
+                    elif "total_track_ids" in zone_data and isinstance(zone_data.get("total_track_ids"), list):
+                        total_count = len(zone_data.get("total_track_ids", []))
+                    else:
+                        counts_dict = zone_data if isinstance(zone_data, dict) else {}
+                        total_count = sum(v for v in counts_dict.values() if isinstance(v, (int, float)))
+                human_text_lines.append(f"\t{zone_name}: {int(total_count)}")
+        else:
+            for cat, count in total_counts_dict.items():
+                if count > 0:
+                    human_text_lines.append(f"\t{cat}: {count}")
+        if alerts:
+            for alert in alerts:
+                human_text_lines.append(f"Alerts: {alert.get('settings', {})} sent @ {current_timestamp}")
+        else:
+            human_text_lines.append("Alerts: None")
         human_text = "\n".join(human_text_lines)
 
         reset_settings = [{"interval_type": "daily", "reset_time": {"value": 9, "time_unit": "hour"}}]
-
         tracking_stat = self.create_tracking_stats(
             total_counts=total_counts,
             current_counts=current_counts,
@@ -432,13 +527,13 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
         tracking_stats.append(tracking_stat)
         return tracking_stats
 
-    def _generate_business_analytics(self, counting_summary: Dict, alerts: Any, config: AbandonedObjectConfig,
+    def _generate_business_analytics(self, counting_summary: Dict, zone_analysis: Dict, alerts: Any, config: AbandonedObjectConfig,
                                     stream_info: Optional[Dict[str, Any]] = None, is_empty=False) -> List[Dict]:
         if is_empty:
             return []
         return []
 
-    def _generate_summary(self, summary: dict, incidents: List, tracking_stats: List, business_analytics: List, alerts: List) -> List[str]:
+    def _generate_summary(self, summary: dict, zone_analysis: Dict, incidents: List, tracking_stats: List, business_analytics: List, alerts: List) -> List[str]:
         lines = {}
         lines["Application Name"] = self.CASE_TYPE
         lines["Application Version"] = self.CASE_VERSION
@@ -450,7 +545,7 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
             lines["Business Analytics:"] = f"\t{business_analytics[0].get('human_text', 'No business analytics detected')}\n"
         if len(incidents) == 0 and len(tracking_stats) == 0 and len(business_analytics) == 0:
             lines["Summary"] = "No Summary Data"
-        return [lines]
+        return ["\n".join(f"{k}: {v}" for k, v in lines.items())]
 
     def _get_track_ids_info(self, detections: list) -> Dict[str, Any]:
         frame_track_ids = {det.get('track_id') for det in detections if det.get('track_id') is not None}
@@ -495,51 +590,81 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
         seconds = round(float(timestamp % 60), 2)
         return f"{hours:02d}:{minutes:02d}:{seconds:.1f}"
 
-    def _get_current_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False, frame_id: Optional[str] = None) -> str:
+    def _get_current_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False, frame_id: Optional[str]=None) -> str:
         if not stream_info:
-            return "00:00:00.00" if precision else "00:00:00"
+            return "00:00:00.00"
         if precision:
             if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
-                start_time = (int(frame_id) if frame_id else stream_info.get("input_settings", {}).get("start_frame", 30)) / stream_info.get("input_settings", {}).get("original_fps", 30)
-                return self._format_timestamp_for_video(start_time)
-            return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
-        if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
-            start_time = (int(frame_id) if frame_id else stream_info.get("input_settings", {}).get("start_frame", 30)) / stream_info.get("input_settings", {}).get("original_fps", 30)
-            return self._format_timestamp_for_video(start_time)
-        stream_time_str = stream_info.get("input_settings", {}).get("stream_info", {}).get("stream_time", "")
-        if stream_time_str:
-            try:
-                timestamp_str = stream_time_str.replace(" UTC", "")
-                dt = datetime.strptime(timestamp_str, "%Y-%m-%d-%H:%M:%S.%f")
-                timestamp = dt.replace(tzinfo=timezone.utc).timestamp()
-                return self._format_timestamp_for_stream(timestamp)
-            except:
-                return self._format_timestamp_for_stream(time.time())
-        return self._format_timestamp_for_stream(time.time())
+                if frame_id:
+                    start_time = int(frame_id)/stream_info.get("input_settings", {}).get("original_fps", 30)
+                else:
+                    start_time = stream_info.get("input_settings", {}).get("start_frame", 30)/stream_info.get("input_settings", {}).get("original_fps", 30)
+                stream_time_str = self._format_timestamp_for_video(start_time)
+                return self._format_timestamp(stream_info.get("input_settings", {}).get("stream_time", "NA"))
+            else:
+                return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
 
-    def _get_start_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False) -> str:
-        if not stream_info:
-            return "00:00:00"
-        if precision:
-            if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
-                return "00:00:00"
-            return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
         if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
-            return "00:00:00"
-        if self._tracking_start_time is None:
+            if frame_id:
+                start_time = int(frame_id)/stream_info.get("input_settings", {}).get("original_fps", 30)
+            else:
+                start_time = stream_info.get("input_settings", {}).get("start_frame", 30)/stream_info.get("input_settings", {}).get("original_fps", 30)
+
+            stream_time_str = self._format_timestamp_for_video(start_time)
+            return self._format_timestamp(stream_info.get("input_settings", {}).get("stream_time", "NA"))
+        else:
             stream_time_str = stream_info.get("input_settings", {}).get("stream_info", {}).get("stream_time", "")
             if stream_time_str:
                 try:
                     timestamp_str = stream_time_str.replace(" UTC", "")
                     dt = datetime.strptime(timestamp_str, "%Y-%m-%d-%H:%M:%S.%f")
-                    self._tracking_start_time = dt.replace(tzinfo=timezone.utc).timestamp()
+                    timestamp = dt.replace(tzinfo=timezone.utc).timestamp()
+                    return self._format_timestamp_for_stream(timestamp)
                 except:
-                    self._tracking_start_time = time.time()
+                    return self._format_timestamp_for_stream(time.time())
             else:
-                self._tracking_start_time = time.time()
-        dt = datetime.fromtimestamp(self._tracking_start_time, tz=timezone.utc)
-        dt = dt.replace(minute=0, second=0, microsecond=0)
-        return dt.strftime('%Y:%m:%d %H:%M:%S')
+                return self._format_timestamp_for_stream(time.time())
+
+    def _get_start_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False) -> str:
+        if not stream_info:
+            return "00:00:00"
+        
+        if precision:
+            if self.start_timer is None:
+                self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+                return self._format_timestamp(self.start_timer)
+            elif stream_info.get("input_settings", {}).get("start_frame", "na") == 1:
+                self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+                return self._format_timestamp(self.start_timer)
+            else:
+                return self._format_timestamp(self.start_timer)
+
+        if self.start_timer is None:
+            self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+            return self._format_timestamp(self.start_timer)
+        elif stream_info.get("input_settings", {}).get("start_frame", "na") == 1:
+            self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+            return self._format_timestamp(self.start_timer)
+        
+        else:
+            if self.start_timer is not None:
+                return self._format_timestamp(self.start_timer)
+
+            if self._tracking_start_time is None:
+                stream_time_str = stream_info.get("input_settings", {}).get("stream_info", {}).get("stream_time", "")
+                if stream_time_str:
+                    try:
+                        timestamp_str = stream_time_str.replace(" UTC", "")
+                        dt = datetime.strptime(timestamp_str, "%Y-%m-%d-%H:%M:%S.%f")
+                        self._tracking_start_time = dt.replace(tzinfo=timezone.utc).timestamp()
+                    except:
+                        self._tracking_start_time = time.time()
+                else:
+                    self._tracking_start_time = time.time()
+
+            dt = datetime.fromtimestamp(self._tracking_start_time, tz=timezone.utc)
+            dt = dt.replace(minute=0, second=0, microsecond=0)
+            return dt.strftime('%Y:%m:%d %H:%M:%S')
 
     def _count_categories(self, detections: list, config: AbandonedObjectConfig) -> dict:
         counts = {}
@@ -592,25 +717,20 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
             return 0.0
         x1_min, y1_min, x1_max, y1_max = l1
         x2_min, y2_min, x2_max, y2_max = l2
-
         x1_min, x1_max = min(x1_min, x1_max), max(x1_min, x1_max)
         y1_min, y1_max = min(y1_min, y1_max), max(y1_min, y1_max)
         x2_min, x2_max = min(x2_min, x2_max), max(x2_min, x2_max)
         y2_min, y2_max = min(y2_min, y2_max), max(y2_min, y2_max)
-
         inter_x_min = max(x1_min, x2_min)
         inter_y_min = max(y1_min, y2_min)
         inter_x_max = min(x1_max, x2_max)
         inter_y_max = min(y1_max, y2_max)
-
         inter_w = max(0.0, inter_x_max - inter_x_min)
         inter_h = max(0.0, inter_y_max - inter_y_min)
         inter_area = inter_w * inter_h
-
         area1 = (x1_max - x1_min) * (y1_max - y1_min)
         area2 = (x2_max - x2_min) * (y2_max - y2_min)
         union_area = area1 + area2 - inter_area
-
         return (inter_area / union_area) if union_area > 0 else 0.0
 
     def _merge_or_register_track(self, raw_id: Any, bbox: Any) -> Any:
@@ -625,7 +745,6 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
                 track_info["last_update"] = now
                 track_info["raw_ids"].add(raw_id)
             return canonical_id
-
         for canonical_id, info in self._canonical_tracks.items():
             if now - info["last_update"] > self._track_merge_time_window:
                 continue
@@ -636,7 +755,6 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
                 info["last_update"] = now
                 info["raw_ids"].add(raw_id)
                 return canonical_id
-
         canonical_id = raw_id
         self._track_aliases[raw_id] = canonical_id
         self._canonical_tracks[canonical_id] = {
@@ -646,8 +764,23 @@ class AbandonedObjectDetectionUseCase(BaseProcessor):
         }
         return canonical_id
 
-    def _format_timestamp(self, timestamp: float) -> str:
-        return datetime.fromtimestamp(timestamp, timezone.utc).strftime('%Y-%m-%d %H:%M:%S UTC')
+    def _format_timestamp(self, timestamp: Any) -> str:
+        if isinstance(timestamp, (int, float)):
+            timestamp = datetime.fromtimestamp(timestamp, timezone.utc).strftime(
+                '%Y-%m-%d-%H:%M:%S.%f UTC'
+            )
+        if not isinstance(timestamp, str):
+            return str(timestamp)
+        if '.' not in timestamp:
+            return timestamp
+        main_part, fractional_and_suffix = timestamp.split('.', 1)
+        if ' ' in fractional_and_suffix:
+            fractional_part, suffix = fractional_and_suffix.split(' ', 1)
+            suffix = ' ' + suffix
+        else:
+            fractional_part, suffix = fractional_and_suffix, ''
+        fractional_part = (fractional_part + '00')[:2]
+        return f"{main_part}.{fractional_part}{suffix}"
 
     def _get_tracking_start_time(self) -> str:
         if self._tracking_start_time is None:
