@@ -23,7 +23,7 @@ import cv2
 import numpy as np
 #import torch
 import re
-from collections import Counter
+from collections import Counter, defaultdict
 #from turbojpeg import TurboJPEG, TJPF_RGB
 os.environ["ORT_LOG_SEVERITY_LEVEL"] = "3"
 # Fast license-plate OCR (replaces EasyOCR)
@@ -116,8 +116,12 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         self._tracked_plate_texts: Dict[Any, str] = {}
         # Containers for text stability & uniqueness
         self._unique_plate_texts: Dict[str, str] = {}  # cleaned_text -> original (longest)
+        # NEW: track-wise frequency of cleaned texts to pick the dominant variant per track
+        self._track_text_counts: Dict[Any, Counter] = defaultdict(Counter)  # track_id -> Counter(cleaned_text -> count)
         # Helper dictionary to keep history of plate texts per track
         self.helper: Dict[Any, List[str]] = {}
+        # Map of track_id -> current dominant plate text
+        self.unique_plate_track: Dict[Any, str] = {}
         self.image_preprocessor = ImagePreprocessor()
         # Fast OCR model (shared across instances)
         if LicensePlateMonitorUseCase._ocr_model is None:
@@ -158,6 +162,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         self._text_history = {}
         self._unique_plate_texts = {}
         self.helper = {}
+        self.unique_plate_track = {}
         self.logger.info("Plate tracking state reset")
 
     def reset_all_tracking(self) -> None:
@@ -180,7 +185,19 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             if not input_bytes:
                 return self.create_error_result("input_bytes (video/image) is required for license plate monitoring",
                                                usecase=self.name, category=self.category, context=context)
+                                               
+            print("--------------------------------------")
+            print("config.alert_config",config.alert_config)
+            print(config)
+            print("--------------------------------------")
             
+            # Normalize alert_config if provided as a plain dict (JS JSON)
+            if isinstance(getattr(config, 'alert_config', None), dict):
+                try:
+                    config.alert_config = AlertConfig(**config.alert_config)  # type: ignore[arg-type]
+                except Exception:
+                    pass
+
             # Initialize OCR extractor if not already done
             if self.ocr_model is None:
                 self.logger.info("Lazy initialisation fallback (should rarely happen)")
@@ -212,8 +229,13 @@ class LicensePlateMonitorUseCase(BaseProcessor):
                 processed_data = apply_category_mapping(processed_data, config.index_to_category)
                 #self.logger.debug("Applied category mapping")
             #print("---------DATA2--------------",processed_data)
-            # Step 3: Filter to target categories
-            processed_data = [d for d in processed_data if d.get('category') in self.target_categories]
+            # Step 3: Filter to target categories (handle dict or list)
+            if isinstance(processed_data, dict):
+                processed_data = processed_data.get("detections", [])
+            # Accept case-insensitive category values and allow overriding via config
+            effective_targets = getattr(config, 'target_categories', self.target_categories) or self.target_categories
+            targets_lower = {str(cat).lower() for cat in effective_targets}
+            processed_data = [d for d in processed_data if str(d.get('category', '')).lower() in targets_lower]
             #self.logger.debug("Applied category filtering")
             
             raw_processed_data = [copy.deepcopy(det) for det in processed_data]
@@ -239,7 +261,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
                 if self.tracker is None:
                     tracker_config = TrackerConfig(
                         track_high_thresh=float(config.confidence_threshold),
-                        track_low_thresh=0.25,
+                        track_low_thresh=max(0.05, float(config.confidence_threshold) / 2),
                         new_track_thresh=float(config.confidence_threshold)
                     )
                     self.tracker = AdvancedTracker(tracker_config)
@@ -293,6 +315,20 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             tracking_stats = tracking_stats_list[0] if tracking_stats_list else {}
             business_analytics = business_analytics_list[0] if business_analytics_list else {}
             summary = summary_list[0] if summary_list else {}
+            # Build LPR_dict (per-track history) and counter (dominant in last 50%)
+            LPR_dict = {}
+            counter = {}
+            for tid, history in self.helper.items():
+                if not history:
+                    continue
+                LPR_dict[str(tid)] = list(history)
+                # dominant from last 50%
+                half = max(1, len(history) // 2)
+                window = history[-half:]
+                from collections import Counter as _Ctr
+                dom, cnt = _Ctr(window).most_common(1)[0]
+                counter[str(tid)] = {"plate": dom, "count": cnt}
+
             agg_summary = {str(frame_number): {
                 "incidents": incidents,
                 "tracking_stats": tracking_stats,
@@ -499,36 +535,57 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         unique_texts: set = set()
         valid_detections: List[Dict[str, Any]] = []
 
-        #print("---------SUMMARY COUNT CATEGORIES",detections)
+        # Group detections by track_id for per-track dominance
+        tracks: Dict[Any, List[Dict[str, Any]]] = {}
         for det in detections:
             if not all(k in det for k in ['category', 'confidence', 'bounding_box']):
                 continue
+            tid = det.get('track_id')
+            if tid is None:
+                # If no track id, treat as its own pseudo-track keyed by bbox
+                tid = (det.get("bounding_box") or det.get("bbox"))
+            tracks.setdefault(tid, []).append(det)
 
-            cat = det.get('category', '')
-            plate_text_raw = det.get('plate_text', '')
-            #print("---------SUMMARY COUNT CATEGORIES PLATE TEXT RAW",plate_text_raw)
-            #print("---------SUMMARY COUNT CATEGORIES PLATE TEXT RAW",det)
-            cleaned_text = plate_text_raw #self._clean_text(plate_text_raw)
+        for tid, dets in tracks.items():
+            # Pick a representative bbox (first occurrence)
+            rep = dets[0]
+            cat = rep.get('category', '')
+            bbox = rep.get('bounding_box')
+            conf = rep.get('confidence')
+            frame_id = rep.get('frame_id')
 
-            # Consider as unique only if meets criteria
-            if cleaned_text and len(cleaned_text) >= self._min_plate_len:
-                unique_texts.add(cleaned_text)
+            # Compute dominant text for this track from last 50% of history
+            dominant_text = None
+            history = self.helper.get(tid, [])
+            if history:
+                half = max(1, len(history) // 2)
+                window = history[-half:]
+                from collections import Counter as _Ctr
+                dominant_text, _ = _Ctr(window).most_common(1)[0]
+            elif rep.get('plate_text'):
+                candidate = self._clean_text(rep.get('plate_text', ''))
+                if self._min_plate_len <= len(candidate) <= 6:
+                    dominant_text = candidate
 
-            valid_detections.append({
-                "bounding_box": det.get("bounding_box"),
-                "category": cat,
-                "confidence": det.get("confidence"),
-                "track_id": det.get('track_id'),
-                "frame_id": det.get("frame_id"),
-                "masks": det.get("masks", []),
-                "plate_text": plate_text_raw
-            })
+            # Fallback to already computed per-track mapping
+            if not dominant_text:
+                dominant_text = self.unique_plate_track.get(tid)
+
+            # Enforce length 5–6 and uniqueness per frame
+            if dominant_text and self._min_plate_len <= len(dominant_text) <= 6:
+                unique_texts.add(dominant_text)
+                valid_detections.append({
+                    "bounding_box": bbox,
+                    "category": cat,
+                    "confidence": conf,
+                    "track_id": rep.get('track_id'),
+                    "frame_id": frame_id,
+                    "masks": rep.get("masks", []),
+                    "plate_text": dominant_text
+                })
 
         counts = {"License_Plate": len(unique_texts)} if unique_texts else {}
 
-        # print("---------SUMMARY COUNT CATEGORIES VALID DETECTIONS",valid_detections)
-        # print(len(unique_texts),'per_cat_count',counts)
-
         return {
             "total_count": len(unique_texts),
             "per_category_count": counts,
@@ -557,20 +614,18 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             category_counts = [f"{count} {cat}" for cat, count in per_category_count.items()]
             detection_text = category_counts[0] + " detected" if len(category_counts) == 1 else f"{', '.join(category_counts[:-1])}, and {category_counts[-1]} detected"
             human_text_lines.append(f"\t- {detection_text}")
-            # Deduplicate plate texts for the current frame while preserving order -
-            seen_cleaned = set()
+            # Show dominant per-track license plates for current frame
+            seen = set()
             display_texts = []
             for det in counting_summary.get("detections", []):
-                raw = det.get("plate_text", "")
-                cleaned = self._clean_text(raw)
-                if cleaned and cleaned not in seen_cleaned:
-                    seen_cleaned.add(cleaned)
-                    # Display the longest raw variant seen in this frame for that cleaned text
-                    longest_variant = max(
-                        [d.get("plate_text", "") for d in counting_summary.get("detections", []) if self._clean_text(d.get("plate_text", "")) == cleaned],
-                        key=len,
-                    )
-                    display_texts.append(longest_variant)
+                t = det.get("track_id")
+                dom = det.get("plate_text")
+                if not dom or not (self._min_plate_len <= len(dom) <= 6):
+                    continue
+                if t in seen:
+                    continue
+                seen.add(t)
+                display_texts.append(dom)
             if display_texts:
                 human_text_lines.append(f"\t- License Plates: {', '.join(display_texts)}")
         else:
@@ -591,36 +646,30 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         human_text = "\n".join(human_text_lines)
         detections = []
         for detection in counting_summary.get("detections", []):
-            raw = detection.get("plate_text", "")
-            cleaned = self._clean_text(raw)
-            # if cleaned and cleaned not in seen_cleaned2:
-            #     seen_cleaned2.add(cleaned)
-            #     # Display the longest raw variant seen in this frame for that cleaned text
-            if cleaned:
-                #print("---------CLEANED-------------",cleaned)
-                plate_text = max(
-                    [d.get("plate_text", "") for d in counting_summary.get("detections", []) if self._clean_text(d.get("plate_text", "")) == cleaned],
-                    key=len,
-                )
-            else:
-                plate_text = "license_plate"
-            
+            dom = detection.get("plate_text", "")
+            if not dom:
+                dom = "license_plate"
             bbox = detection.get("bounding_box", {})
             category = detection.get("category", "license_plate")
-            #plate_text = detection.get("plate_text", "")
             segmentation = detection.get("masks", detection.get("segmentation", detection.get("mask", [])))
-            detection_obj = self.create_detection_object(category, bbox, segmentation=None, plate_text=plate_text)
+            detection_obj = self.create_detection_object(category, bbox, segmentation=None, plate_text=dom)
             detections.append(detection_obj)
         
         alert_settings = []
-        if config.alert_config and hasattr(config.alert_config, 'alert_type'):
+        # Build alert settings tolerating dict or dataclass for alert_config
+        if config.alert_config:
+            alert_cfg = config.alert_config
+            alert_type = getattr(alert_cfg, 'alert_type', None) if not isinstance(alert_cfg, dict) else alert_cfg.get('alert_type')
+            alert_value = getattr(alert_cfg, 'alert_value', None) if not isinstance(alert_cfg, dict) else alert_cfg.get('alert_value')
+            count_thresholds = getattr(alert_cfg, 'count_thresholds', None) if not isinstance(alert_cfg, dict) else alert_cfg.get('count_thresholds')
+            alert_type = alert_type if isinstance(alert_type, list) else (list(alert_type) if alert_type is not None else ['Default'])
+            alert_value = alert_value if isinstance(alert_value, list) else (list(alert_value) if alert_value is not None else ['JSON'])
             alert_settings.append({
-                "alert_type": getattr(config.alert_config, 'alert_type', ['Default']),
+                "alert_type": alert_type,
                 "incident_category": self.CASE_TYPE,
-                "threshold_level": config.alert_config.count_thresholds if hasattr(config.alert_config, 'count_thresholds') else {},
+                "threshold_level": count_thresholds or {},
                 "ascending": True,
-                "settings": {t: v for t, v in zip(getattr(config.alert_config, 'alert_type', ['Default']), 
-                                                  getattr(config.alert_config, 'alert_value', ['JSON']))}
+                "settings": {t: v for t, v in zip(alert_type, alert_value)}
             })
         
         if alerts:
@@ -665,27 +714,32 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         if not config.alert_config:
             return alerts
 
-        if hasattr(config.alert_config, 'count_thresholds') and config.alert_config.count_thresholds:
-            for category, threshold in config.alert_config.count_thresholds.items():
+        # Extract thresholds regardless of dict/dataclass
+        _alert_cfg = config.alert_config
+        _thresholds = getattr(_alert_cfg, 'count_thresholds', None) if not isinstance(_alert_cfg, dict) else _alert_cfg.get('count_thresholds')
+        _types = getattr(_alert_cfg, 'alert_type', None) if not isinstance(_alert_cfg, dict) else _alert_cfg.get('alert_type')
+        _values = getattr(_alert_cfg, 'alert_value', None) if not isinstance(_alert_cfg, dict) else _alert_cfg.get('alert_value')
+        _types = _types if isinstance(_types, list) else (list(_types) if _types is not None else ['Default'])
+        _values = _values if isinstance(_values, list) else (list(_values) if _values is not None else ['JSON'])
+        if _thresholds:
+            for category, threshold in _thresholds.items():
                 if category == "all" and total_detections > threshold:
                     alerts.append({
-                        "alert_type": getattr(config.alert_config, 'alert_type', ['Default']),
+                        "alert_type": _types,
                         "alert_id": f"alert_{category}_{frame_key}",
                         "incident_category": self.CASE_TYPE,
                         "threshold_level": threshold,
                         "ascending": get_trend(self._ascending_alert_list),
-                        "settings": {t: v for t, v in zip(getattr(config.alert_config, 'alert_type', ['Default']), 
-                                                          getattr(config.alert_config, 'alert_value', ['JSON']))}
+                        "settings": {t: v for t, v in zip(_types, _values)}
                     })
                 elif category in per_category_count and per_category_count[category] > threshold:
                     alerts.append({
-                        "alert_type": getattr(config.alert_config, 'alert_type', ['Default']),
+                        "alert_type": _types,
                         "alert_id": f"alert_{category}_{frame_key}",
                         "incident_category": self.CASE_TYPE,
                         "threshold_level": threshold,
                         "ascending": get_trend(self._ascending_alert_list),
-                        "settings": {t: v for t, v in zip(getattr(config.alert_config, 'alert_type', ['Default']), 
-                                                          getattr(config.alert_config, 'alert_value', ['JSON']))}
+                        "settings": {t: v for t, v in zip(_types, _values)}
                     })
         return alerts
 
@@ -750,14 +804,19 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             human_text = "\n".join(human_text_lines)
 
             alert_settings = []
-            if config.alert_config and hasattr(config.alert_config, 'alert_type'):
+            if config.alert_config:
+                _alert_cfg = config.alert_config
+                _types = getattr(_alert_cfg, 'alert_type', None) if not isinstance(_alert_cfg, dict) else _alert_cfg.get('alert_type')
+                _values = getattr(_alert_cfg, 'alert_value', None) if not isinstance(_alert_cfg, dict) else _alert_cfg.get('alert_value')
+                _thresholds = getattr(_alert_cfg, 'count_thresholds', None) if not isinstance(_alert_cfg, dict) else _alert_cfg.get('count_thresholds')
+                _types = _types if isinstance(_types, list) else (list(_types) if _types is not None else ['Default'])
+                _values = _values if isinstance(_values, list) else (list(_values) if _values is not None else ['JSON'])
                 alert_settings.append({
-                    "alert_type": getattr(config.alert_config, 'alert_type', ['Default']),
+                    "alert_type": _types,
                     "incident_category": self.CASE_TYPE,
-                    "threshold_level": config.alert_config.count_thresholds if hasattr(config.alert_config, 'count_thresholds') else {},
+                    "threshold_level": _thresholds or {},
                     "ascending": True,
-                    "settings": {t: v for t, v in zip(getattr(config.alert_config, 'alert_type', ['Default']), 
-                                                      getattr(config.alert_config, 'alert_value', ['JSON']))}
+                    "settings": {t: v for t, v in zip(_types, _values)}
                 })
         
             event = self.create_incident(
@@ -817,115 +876,54 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             self._current_frame_track_ids[cat].add(canonical_id)
 
     def _update_plate_texts(self, detections: List[Dict]):
-        """Update plate text history, compute dominant text per track and merge duplicate tracks.
-
-        For each ``track_id`` we keep a running list (``self.helper``) of all recognised
-        plate texts *in the order they were seen*.  After every frame we:
-
-        1.  Derive the **dominant** plate text for every track by looking only at the
-            **last 50 %** of the observations for that track and selecting the most
-            frequent value.  When multiple values are tied we take the one that
-            appears **latest** in the observation window (i.e. closest to the
-            current frame).
-        2.  Merge different ``track_id``\ s that resolve to the *same* dominant
-            plate text (perfect textual match).  The first track_id that produced
-            the text becomes the canonical id; all other tracks are remapped to
-            this id for the remainder of the processing.
-
-        The method updates the following internal structures so that subsequent
-        counting/summary steps see the correct data:
-        - ``self._tracked_plate_texts``   →  mapping of canonical *track_id → text*
-        - ``self._unique_plate_texts``    →  mapping of *cleaned_text → longest_raw*
-        - ``detections`` list (in-place)  →  each detection now carries the
-          canonical ``track_id`` and the dominant ``plate_text``.
-        """
-        
-        def _select_dominant(text_list: List[str]) -> str:
-            """Return dominant element from the last 50 % of *text_list*.
-
-            When there is a tie we return the **latest** occurring element among
-            the tied ones.
-            """
-            if not text_list:
-                return ""
-            half_idx = len(text_list) // 2  # start of the last 50 %
-            window = text_list[half_idx:]
-            counts = Counter(window)
-            max_freq = max(counts.values())
-            # All items with max frequency
-            candidates = {txt for txt, cnt in counts.items() if cnt == max_freq}
-            if len(candidates) == 1:
-                return next(iter(candidates))
-            # Tie-breaker – iterate backwards to find latest candidate
-            for txt in reversed(window):
-                if txt in candidates:
-                    return txt
-            # Fallback (should not happen)
-            return window[-1]
-
-        # ------------------------------------------------------------------
-        # 1)  Collect plate texts observed in this frame --------------------
-        # ------------------------------------------------------------------
+        """Update set of seen plate texts and track the longest plate_text per track_id."""
         for det in detections:
-            raw_text = det.get("plate_text") or ""
-            if not raw_text:
+            raw_text = det.get('plate_text')
+            track_id = det.get('track_id')
+            if not raw_text or track_id is None:
                 continue
+
             cleaned = self._clean_text(raw_text)
-            if len(cleaned) < self._min_plate_len:
-                # Too short – ignore (length & alphanumeric checks stay intact)
+
+            # Enforce plate length 5 or 6 characters ("greater than 4 and less than 7")
+            if not (self._min_plate_len <= len(cleaned) <= 6):
                 continue
-            track_id = det.get("track_id")
-            if track_id is None:
-                continue
-            # Keep full raw_text – consumers may prefer longest raw variant
-            self.helper.setdefault(track_id, []).append(raw_text)
 
-        # ------------------------------------------------------------------
-        # 2)  Compute dominant text per track --------------------------------
-        # ------------------------------------------------------------------
-        track_to_text: Dict[Any, str] = {}
-        for t_id, text_list in self.helper.items():
-            dominant_raw = _select_dominant(text_list)
-            if dominant_raw:
-                track_to_text[t_id] = dominant_raw
+            # Append to per-track rolling history (keep reasonable size)
+            history = self.helper.get(track_id)
+            if history is None:
+                history = []
+                self.helper[track_id] = history
+            history.append(cleaned)
+            if len(history) > 200:
+                del history[: len(history) - 200]
 
-        # ------------------------------------------------------------------
-        # 3)  Merge tracks that share the same dominant text -----------------
-        # ------------------------------------------------------------------
-        text_to_canonical: Dict[str, Any] = {}
-        track_id_remap: Dict[Any, Any] = {}
-        for t_id, text in track_to_text.items():
-            cleaned = self._clean_text(text)
-            if cleaned in text_to_canonical:
-                # Duplicate plate → map to existing canonical id
-                canonical_id = text_to_canonical[cleaned]
-                track_id_remap[t_id] = canonical_id
-            else:
-                text_to_canonical[cleaned] = t_id
+            # Update per-track frequency counter (all-time)
+            self._track_text_counts[track_id][cleaned] += 1
 
-        # Update internal dictionaries -------------------------------------
-        self._tracked_plate_texts = {track_id_remap.get(t_id, t_id): txt for t_id, txt in track_to_text.items()}
-        # Maintain unique cleaned → longest_raw mapping
-        for raw in track_to_text.values():
-            cleaned = self._clean_text(raw)
-            longest = self._unique_plate_texts.get(cleaned, "")
-            if len(raw) > len(longest):
-                self._unique_plate_texts[cleaned] = raw
+            # Update consecutive frame counter for stability across whole video
+            self._text_history[cleaned] = self._text_history.get(cleaned, 0) + 1
 
-        # ------------------------------------------------------------------
-        # 4)  Apply remapping & dominant text to current detections ----------
-        # ------------------------------------------------------------------
-        for det in detections:
-            track_id = det.get("track_id")
-            if track_id is None:
-                continue
-            canonical_id = track_id_remap.get(track_id, track_id)
-            det["track_id"] = canonical_id
-            dominant_text = self._tracked_plate_texts.get(canonical_id, det.get("plate_text"))
-            det["plate_text"] = dominant_text
+            # Once stable, decide dominant text from LAST 50% of history
+            if self._text_history[cleaned] >= self._stable_frames_required:
+                half = max(1, len(history) // 2)
+                window = history[-half:]
+                from collections import Counter as _Ctr
+                dominant, _ = _Ctr(window).most_common(1)[0]
 
-        # No need for consecutive-frame stability counters any more – remove
-        self._text_history = {}
+                # Update per-track mapping to dominant
+                self._tracked_plate_texts[track_id] = dominant
+                self.unique_plate_track[track_id] = dominant
+
+                # Maintain global unique mapping with dominant only
+                if dominant not in self._unique_plate_texts:
+                    self._unique_plate_texts[dominant] = dominant
+
+        # Reset counters for texts NOT seen in this frame (to preserve stability requirement)
+        current_frame_texts = {self._clean_text(det.get('plate_text', '')) for det in detections if det.get('plate_text')}
+        for t in list(self._text_history.keys()):
+            if t not in current_frame_texts:
+                self._text_history[t] = 0
 
     def get_total_counts(self):
         """Return total unique license plate texts encountered so far."""
