@@ -24,7 +24,7 @@ class WeaponDetectionConfig(BaseConfig):
     smoothing_window_size: int = 20
     smoothing_cooldown_frames: int = 5
     smoothing_confidence_range_factor: float = 0.5
-    confidence_threshold: float = 0.6
+    confidence_threshold: float = 0.3
     weapon_categories: List[str] = field(default_factory=lambda: ['billete', 'bluntweapon', 'glass', 'gun', 'knife', 'monedero', 'pistol', 'smartphone', 'tarjeta'])
     target_weapon_categories: List[str] = field(default_factory=lambda: ['bluntweapon', 'glass', 'gun', 'knife', 'monedero', 'pistol', 'tarjeta'])
     alert_config: Optional[AlertConfig] = None
@@ -72,6 +72,7 @@ class WeaponDetectionUseCase(BaseProcessor):
         self._track_merge_time_window: float = 7.0
         self._ascending_alert_list: List[int] = []
         self.current_incident_end_timestamp: str = "N/A"
+        self.start_timer = None
 
     def process(self, data: Any, config: ConfigProtocol, context: Optional[ProcessingContext] = None,
                 stream_info: Optional[Dict[str, Any]] = None) -> ProcessingResult:
@@ -96,14 +97,7 @@ class WeaponDetectionUseCase(BaseProcessor):
             processed_data = apply_category_mapping(processed_data, config.index_to_category)
             self.logger.debug("Applied category mapping")
 
-        if config.target_weapon_categories:
-            processed_data = [d for d in processed_data if d.get('category') in self.target_categories]
-            self.logger.debug("Applied category filtering")
-
-        # Alert for detected weapons
-        for detection in processed_data:
-            if detection.get('category') in config.target_weapon_categories:
-                self.logger.warning(f"ALERT: {detection.get('category')} detected at {self._get_current_timestamp_str(stream_info)}")
+        processed_data = self._check_weapon_objects(processed_data, config)
 
         if config.enable_smoothing:
             if self.smoothing_tracker is None:
@@ -173,6 +167,14 @@ class WeaponDetectionUseCase(BaseProcessor):
         )
         return result
 
+    def _check_weapon_objects(self, data: List[Dict], config: WeaponDetectionConfig) -> List[Dict]:
+        weapon_data = []
+        for det in data:
+            if det.get('category') in config.target_weapon_categories:
+                det['category'] = 'weapon'
+                weapon_data.append(det)
+        return weapon_data
+
     def _check_alerts(self, summary: dict, frame_number: Any, config: WeaponDetectionConfig) -> List[Dict]:
         def get_trend(data, lookback=900, threshold=0.6):
             window = data[-lookback:] if len(data) >= lookback else data
@@ -335,6 +337,7 @@ class WeaponDetectionUseCase(BaseProcessor):
                 "settings": {t: v for t, v in zip(getattr(config.alert_config, 'alert_type', ['Default']),
                                                 getattr(config.alert_config, 'alert_value', ['JSON']))}
             })
+
         weapon_counts = {cat: per_category_count.get(cat, 0) for cat in config.target_weapon_categories}
 
         human_text_lines = [f"Tracking Statistics:"]
@@ -342,7 +345,7 @@ class WeaponDetectionUseCase(BaseProcessor):
         if any(count > 0 for count in weapon_counts.values()):
             for cat, count in weapon_counts.items():
                 if count > 0:
-                    human_text_lines.append(f"\t- {count} {cat.capitalize()} incident(s) detected")
+                    human_text_lines.append(f"\t- {count} weapons(s) detected")
         else:
             human_text_lines.append(f"\t- No weapon[s] detected")
 
@@ -350,7 +353,7 @@ class WeaponDetectionUseCase(BaseProcessor):
         human_text_lines.append(f"TOTAL SINCE {start_timestamp}")
         for cat, count in total_counts_dict.items():
             if count > 0:
-                human_text_lines.append(f"\t{count} {cat} [s] Detected")
+                human_text_lines.append(f"\t{count} weapon[s] Detected")
         human_text_lines.append("")
         human_text_lines.append("Alerts: None" if not alerts else f"Alerts: {alerts[0].get('settings', {})} sent @ {current_timestamp}")
         human_text = "\n".join(human_text_lines)
@@ -378,7 +381,7 @@ class WeaponDetectionUseCase(BaseProcessor):
             return []
         return []
 
-    def _generate_summary(self, summary: dict, incidents: List, tracking_stats: List, business_analytics: List, alerts: List) -> List[Dict]:
+    def _generate_summary(self, counting_summary: Dict, incidents: List, tracking_stats: List, business_analytics: List, alerts: List) -> List[Dict]:
         lines = {}
         lines["Application Name"] = self.CASE_TYPE
         lines["Application Version"] = self.CASE_VERSION
@@ -425,22 +428,20 @@ class WeaponDetectionUseCase(BaseProcessor):
     def get_total_counts(self):
         return {cat: len(ids) for cat, ids in getattr(self, '_per_category_total_track_ids', {}).items()}
 
-    def _format_timestamp_for_video(self, timestamp: float) -> str:
-        """Format timestamp for video chunks (HH:MM:SS.ms format)."""
-        hours = int(timestamp // 3600)
-        minutes = int((timestamp % 3600) // 60)
-        seconds = round(float(timestamp % 60),2)
-        return f"{hours:02d}:{minutes:02d}:{seconds:.1f}"
-
     def _format_timestamp_for_stream(self, timestamp: float) -> str:
         dt = datetime.fromtimestamp(timestamp, tz=timezone.utc)
         return dt.strftime('%Y:%m:%d %H:%M:%S')
 
+    def _format_timestamp_for_video(self, timestamp: float) -> str:
+        hours = int(timestamp // 3600)
+        minutes = int((timestamp % 3600) // 60)
+        seconds = round(float(timestamp % 60), 2)
+        return f"{hours:02d}:{minutes:02d}:{seconds:.1f}"
+
     def _get_current_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False, frame_id: Optional[str]=None) -> str:
         """Get formatted current timestamp based on stream type."""
         if not stream_info:
             return "00:00:00.00"
-        # is_video_chunk = stream_info.get("input_settings", {}).get("is_video_chunk", False)
         if precision:
             if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
                 if frame_id:
@@ -448,30 +449,26 @@ class WeaponDetectionUseCase(BaseProcessor):
                 else:
                     start_time = stream_info.get("input_settings", {}).get("start_frame", 30)/stream_info.get("input_settings", {}).get("original_fps", 30)
                 stream_time_str = self._format_timestamp_for_video(start_time)
-                return stream_time_str
+                return self._format_timestamp(stream_info.get("input_settings", {}).get("stream_time", "NA"))
             else:
                 return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
 
         if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
-                if frame_id:
-                    start_time = int(frame_id)/stream_info.get("input_settings", {}).get("original_fps", 30)
-                else:
-                    start_time = stream_info.get("input_settings", {}).get("start_frame", 30)/stream_info.get("input_settings", {}).get("original_fps", 30)
-                stream_time_str = self._format_timestamp_for_video(start_time)
-                return stream_time_str
+            if frame_id:
+                start_time = int(frame_id)/stream_info.get("input_settings", {}).get("original_fps", 30)
+            else:
+                start_time = stream_info.get("input_settings", {}).get("start_frame", 30)/stream_info.get("input_settings", {}).get("original_fps", 30)
+            stream_time_str = self._format_timestamp_for_video(start_time)
+            return self._format_timestamp(stream_info.get("input_settings", {}).get("stream_time", "NA"))
         else:
-            # For streams, use stream_time from stream_info
             stream_time_str = stream_info.get("input_settings", {}).get("stream_info", {}).get("stream_time", "")
             if stream_time_str:
-                # Parse the high precision timestamp string to get timestamp
                 try:
-                    # Remove " UTC" suffix and parse
                     timestamp_str = stream_time_str.replace(" UTC", "")
                     dt = datetime.strptime(timestamp_str, "%Y-%m-%d-%H:%M:%S.%f")
                     timestamp = dt.replace(tzinfo=timezone.utc).timestamp()
                     return self._format_timestamp_for_stream(timestamp)
                 except:
-                    # Fallback to current time if parsing fails
                     return self._format_timestamp_for_stream(time.time())
             else:
                 return self._format_timestamp_for_stream(time.time())
@@ -480,37 +477,66 @@ class WeaponDetectionUseCase(BaseProcessor):
         """Get formatted start timestamp for 'TOTAL SINCE' based on stream type."""
         if not stream_info:
             return "00:00:00"
+        
         if precision:
-            if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
-                return "00:00:00"
+            if self.start_timer is None:
+                self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+                return self._format_timestamp(self.start_timer)
+            elif stream_info.get("input_settings", {}).get("start_frame", "na") == 1:
+                self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+                return self._format_timestamp(self.start_timer)
             else:
-                return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
+                return self._format_timestamp(self.start_timer)
 
-        if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
-            # If video format, start from 00:00:00
-            return "00:00:00"
+        if self.start_timer is None:
+            self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+            return self._format_timestamp(self.start_timer)
+        elif stream_info.get("input_settings", {}).get("start_frame", "na") == 1:
+            self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+            return self._format_timestamp(self.start_timer)
+        
         else:
-            # For streams, use tracking start time or current time with minutes/seconds reset
+            if self.start_timer is not None:
+                return self._format_timestamp(self.start_timer)
+
             if self._tracking_start_time is None:
-                # Try to extract timestamp from stream_time string
                 stream_time_str = stream_info.get("input_settings", {}).get("stream_info", {}).get("stream_time", "")
                 if stream_time_str:
                     try:
-                        # Remove " UTC" suffix and parse
                         timestamp_str = stream_time_str.replace(" UTC", "")
                         dt = datetime.strptime(timestamp_str, "%Y-%m-%d-%H:%M:%S.%f")
                         self._tracking_start_time = dt.replace(tzinfo=timezone.utc).timestamp()
                     except:
-                        # Fallback to current time if parsing fails
                         self._tracking_start_time = time.time()
                 else:
                     self._tracking_start_time = time.time()
 
             dt = datetime.fromtimestamp(self._tracking_start_time, tz=timezone.utc)
-            # Reset minutes and seconds to 00:00 for "TOTAL SINCE" format
             dt = dt.replace(minute=0, second=0, microsecond=0)
             return dt.strftime('%Y:%m:%d %H:%M:%S')
 
+    def _format_timestamp(self, timestamp: Any) -> str:
+        """Format a timestamp so that exactly two digits follow the decimal point (milliseconds)."""
+        if isinstance(timestamp, (int, float)):
+            timestamp = datetime.fromtimestamp(timestamp, timezone.utc).strftime(
+                '%Y-%m-%d-%H:%M:%S.%f UTC'
+            )
+
+        if not isinstance(timestamp, str):
+            return str(timestamp)
+
+        if '.' not in timestamp:
+            return timestamp
+
+        main_part, fractional_and_suffix = timestamp.split('.', 1)
+        if ' ' in fractional_and_suffix:
+            fractional_part, suffix = fractional_and_suffix.split(' ', 1)
+            suffix = ' ' + suffix
+        else:
+            fractional_part, suffix = fractional_and_suffix, ''
+        fractional_part = (fractional_part + '00')[:2]
+        return f"{main_part}.{fractional_part}{suffix}"
+
     def _count_categories(self, detections: list, config: WeaponDetectionConfig) -> dict:
         counts = {}
         for det in detections:
@@ -534,7 +560,7 @@ class WeaponDetectionUseCase(BaseProcessor):
     def _extract_predictions(self, detections: list) -> List[Dict[str, Any]]:
         return [
             {
-                "category": det.get("category", "unknown"),
+                "category": "Weapon" if det.get("category", "unknown") == "pistol" else det.get("category", "unknown"),
                 "confidence": det.get("confidence", 0.0),
                 "bounding_box": det.get("bounding_box", {})
             }
@@ -609,9 +635,6 @@ class WeaponDetectionUseCase(BaseProcessor):
         }
         return canonical_id
 
-    def _format_timestamp(self, timestamp: float) -> str:
-        return datetime.fromtimestamp(timestamp, timezone.utc).strftime('%Y-%m-%d %H:%M:%S UTC')
-
     def _get_tracking_start_time(self) -> str:
         if self._tracking_start_time is None:
             return "N/A"
